#ifndef __TITLE_GEN_SECTION__
#define __TITLE_GEN_SECTION__

#include <list>
#include <deque>
#include <map>
#include <set>
#include "hash_map.hpp"
#include "nlp_string.h"
#include <iostream>
#include <fstream>


using namespace std;


class Word;
class Sentence;
class Section;

typedef list <Section*>				Section_list_t;
typedef deque <Section*>			Section_dq_t;
typedef list <Sentence*>			Sentence_list_t;
typedef list <String>				Unigram_list_t;
typedef list <Word*>				Word_list_t;
typedef deque <Word*>				Word_dq_t;
typedef list <Word_list_t*>			Ngram_list_t;
typedef set <String>				Unigram_set_t;
typedef hash_map <String, long>		Unigram_to_Count_map_t;
typedef hash_map <String, double>	FrequencyRank_By_Stem_map_t;
typedef hash_map <String, long>		Ngram_to_Count_map_t;

typedef String (*WordRenderer_t)(Word& _rWord);



//======================================================
class Word
{
	private:
		static void LoadStopwords ();
		static void LoadAuxiliaryWords ();
		static bool IsStopWord (const String &_sWord);
		static bool IsAuxiliaryWord (const String &_sWord);

	public:
		static Unigram_set_t	set_Stopwords;
		static Unigram_set_t	set_AuxiliaryWords;
		static Word*			p_StartWord;
		static Word*			p_EndWord;
    
		String	s_Word;
		String	s_Stem;
		String	s_POSTag;
		bool	b_IsStopWord;
		bool	b_IsAuxiliaryWord;
		bool	b_IsContentWord;
		bool	b_StartWord;
		bool	b_EndWord;

		Word (const char* _zWord);

		bool operator== (const Word& _w) const;
		bool operator!= (const Word& _w) const;

		static Word* GetStartWord (void);
		static Word* GetEndWord (void);
		static String GetWordText (Word& _rWord);
		static String GetWordTagText (Word& _rWord);
		static String GetTagText (Word& _rWord);
};

ostream& operator<< (ostream& _ofs, const Word& _rWord);
ostream& operator<< (ostream& _ofs, const Word_dq_t& _dqWords);
ostream& operator<< (ostream& _ofs, const Word_list_t& _lstWords);



//======================================================
class Sentence
{
	private:
		// use parse tree here instead...
		
	public:
		Word_list_t		lst_Words;
		Unigram_set_t	set_Stems;
		String 			s_Text;

		Sentence (const char* _zText);
		~Sentence (void);

		Word_list_t& GetWords (void);
		const char* GetText (void);
		String GetTaggedText (void);
};



//======================================================
#define USE_WORD	'w'
#define USE_STEM	's'

class Section
{
	private:
		static Unigram_to_Count_map_t		map_StemToSectionCount;
		static Unigram_to_Count_map_t		map_WordToSectionCount;
		static Unigram_to_Count_map_t		map_Stem_to_First_Occurrence_by_Section;
		Unigram_to_Count_map_t				map_Stem_to_Count;
		Unigram_to_Count_map_t				map_Word_to_Count;
		FrequencyRank_By_Stem_map_t			map_FrequencyRankByStem;
		Unigram_to_Count_map_t				map_Stem_to_First_Occurrence_by_Word;
		Unigram_to_Count_map_t				map_Stem_to_First_Occurrence_by_Sentence;
		Ngram_to_Count_map_t				map_NP_to_Corpus_Count;
		Ngram_to_Count_map_t				map_NP_to_Count;
		
		int 		i_TotalUnigrams;
 		int             i_Depth;
		
		void ParseBody (void);

	public:
		static Section_list_t	lst_AllSections;
		static int				i_TotalSections;
		static Section*			p_Root;

		Section_list_t			lst_Children;
		Section*				p_Parent;
		Section*				p_PreviousSection;
		Section*				p_PreviousSibling;
		Section*				p_NextSibling;
		Sentence*				p_Title;
		Sentence_list_t			lst_Sentences;
		Word_dq_t				dq_UniqueWords;

		Section (void);
		Section (const char* _zTitle, const char* _zBody);
		~Section (void);

		void AddChild (Section* _pChild);
		Section* GetRoot (void);
		void SetText (const char* _zTitle, const char* _zBody);
		void CreateUniqueWordList (void);

		void BuildRootStructures (void);
		void BuildFirstSectionWithStemMap (void);
		void BuildNPCounts (void);
		void BuildUnigramToSectionCounts (void);

		Unigram_list_t CapitalizeTitle (Word_list_t _lstTitleWords);

		bool IsRoot (void);
		Section_list_t* GetAllSections (void);
		void GetDescendants (Section_list_t* _rlstSections);
		Word_list_t GetBodyWords (void);	

		bool HasUnigram (const char* _zUnigram, bool _bUseStem);
		int GetUnigramOccurrence (const char* _zUnigram, bool _bUseStem);
		int GetNumberOfSectionsWithUnigram (const char* _zUnigram, bool _bUseStem);
		int GetTotalUnigramOccurrences (void);
		long GetFirstOccurrenceByWord (const char* _zStem);
		long GetFirstOccurrenceBySentence (const char* _zStem);
		long GetFirstOccurrenceBySection (const char* _zStem);
		int GetNPCount (const char* _zNP);
		set<String> GetBodyNPs (void);
		int GetCorpusNPCount (const char* _zNP);
		bool PreviousSiblingHasStem (const char* _zStem);
		bool NextSiblingHasStem (const char* _zStem);
		int GetDepth( void );

		static Unigram_to_Count_map_t* GetAllWords (void);
};




#endif
